/*
 * atom_bondedparticleinfo.hpp
 *
 *  Created on: Oct 19, 2009
 *      Author: heber
 */

#ifndef ATOM_BONDEDPARTICLEINFO_HPP_
#define ATOM_BONDEDPARTICLEINFO_HPP_


using namespace std;

/*********************************************** includes ***********************************/

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//#include "atom_observable.hpp"
#include "Bond/bond.hpp"

#include <list>
#include <vector>

/****************************************** forward declarations *****************************/

class BondedParticle;

typedef std::list<bond::ptr > BondList;

/********************************************** declarations *******************************/

class BondedParticleInfo // : public virtual AtomObservable // must be virtual(!)
{
  friend class BondedParticle;
public:

  virtual ~BondedParticleInfo() {}

  /** Pushes back another step in all trajectory vectors.
   *
   * This allows to extend all trajectories contained in different classes
   * consistently. This is implemented by the topmost class which calls the
   * real functions, \sa AppendTrajectoryStep(), by all necessary subclasses.
   */
  virtual void UpdateStep(const unsigned int _step)=0;

  /** Pops the last step in all trajectory vectors.
   *
   * This allows to decrease all trajectories contained in different classes
   * by one consistently. This is implemented by the topmost class which calls
   * the real functions, \sa removeTrajectorySteps(), by all necessary subclasses.
   *
   * \param _firststep first step in interval to be removed
   * \param _laststep last step in interval to be removed
   */
  virtual void removeSteps(const unsigned int _firststep, const unsigned int _laststep)=0;

  /** Const accessor to ListOfBonds of WorldTime::CurrentTime.
   *
   * @return ListOfBonds[WorldTime::CurrentTime]
   */
  const BondList& getListOfBonds() const;

  /** Const Accessor ListOfBonds of any present time step.
   *
   * @param _step time step to access
   * @return ListOfBonds[_step].
   */
  const BondList& getListOfBondsAtStep(unsigned int _step) const;

  /** Const getter for the MaxOrder property at current time step.
   *
   * @return MaxOrder[Current world time step].
   */
  const unsigned char& getMaxOrder() const;

  /** Const getter for the MaxOrder property at given \a _step.
   *
   * @param _step time step to access
   * @return MaxOrder[_step].
   */
  const unsigned char& getMaxOrder(const unsigned int _step) const;

  /** Setter for the MaxOrder property at current time step.
   *
   * @param _value value to set for MaxOrder at current time step
   */
  void setMaxOrder(const unsigned char _value);

  /**Setter for the MaxOrder property at given \a _step.
   *
   * @param _step time step to access
   * @param _value value to set for MaxOrder at the desired time step
   * @return MaxOrder[_step].
   */
  void setMaxOrder(const unsigned int _step, const unsigned char _value);

  /** Const getter for the AdaptiveOrder property at current time step.
   *
   * @return AdaptiveOrder[Current world time step].
   */
  const unsigned char& getAdaptiveOrder() const;

  /** Const getter for the AdaptiveOrder property at given \a _step.
   *
   * @param _step time step to access
   * @return AdaptiveOrder[_step].
   */
  const unsigned char& getAdaptiveOrder(const unsigned int _step) const;

  /** Setter for the AdaptiveOrder property at current time step.
   *
   * @param _value value to set for AdaptiveOrder at current time step
   */
  void setAdaptiveOrder(const unsigned char _value);

  /** Setter for the AdaptiveOrder property at given \a _step.
   *
   * @param _step time step to access
   * @param _value value to set for AdaptiveOrder at the desired time step
   * @return AdaptiveOrder[_step].
   */
  void setAdaptiveOrder(const unsigned int _step, const unsigned char _value);

protected:
  /** Function used by this and inheriting classes to extend the ListOfBonds
   * vector.
   */
  void AppendTrajectoryStep(const unsigned int _step);

  /** Function used by this and inheriting classes to reduce the ListOfBonds
   * vector by one.
   *
   * \param _firststep first step in interval to be removed
   * \param _laststep last step in interval to be removed
   */
  void removeTrajectorySteps(const unsigned int _firststep, const unsigned int _laststep);

  typedef std::map<unsigned int, BondList> BondTrajectory_t;
  BondTrajectory_t ListOfBonds; //!< list of all bonds
  static BondList emptyList;  //!< empty list to return when step is not present

  typedef std::map<unsigned int, unsigned char> OrderTrajectory_t;
  OrderTrajectory_t AdaptiveOrder;  //!< current present bond order at site (0 means "not set")
  OrderTrajectory_t MaxOrder;  //!< desired maximum order of this atom (0 means "not set")
};


#endif /* ATOM_BONDEDPARTICLEINFO_HPP_ */
